//
// Copyright (c) 2012, MindFusion LLC - Bulgaria.
//

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

using MindFusion.Drawing;

using Orientation = MindFusion.Diagramming.Orientation;
using Pen = MindFusion.Drawing.Pen;
using SolidBrush = MindFusion.Drawing.SolidBrush;
using ColumnStyle = MindFusion.Diagramming.ColumnStyle;


namespace MindFusion.Diagramming.WinForms.Samples.CS.SWDesigner
{
	/// <summary>
	/// Summary description for DiagramForm.
	/// </summary>
	public partial class DiagramForm : Form
	{
		public DiagramForm()
		{
			InitializeComponent();

			_cbZoom.SelectedIndex = 3;

			diagram.ShowDisabledHandles = false;
			diagram.Selection.Style = SelectionStyle.Frame;
		}

		public void Save()
		{
			if (diagram.Dirty && _diagram != null)
				_diagram.Data = diagram.SaveToString(SaveToStringFormat.Base64, true);
		}

		private void DiagramForm_Closing(object sender, CancelEventArgs e)
		{
			Save();
		}

		private void _cbZoom_SelectedIndexChanged(object sender, EventArgs e)
		{
			short[] zooms = new short[] { 10, 25, 50, 100, 150 };

			if (_cbZoom.SelectedIndex == -1)
				return;

			diagramView.ZoomFactor = zooms[_cbZoom.SelectedIndex];
		}

		private void _btnNewClass_Click(object sender, EventArgs e)
		{
			TableNode t = new TableNode(diagram);
			t.Bounds = new RectangleF(10, 10, 30, diagram.TableCaptionHeight + 2);
			t.Caption = "Class";
            t.Brush = new SolidBrush(Color.FromArgb(150, Color.FromArgb(97, 106, 127)));
			t.Columns[0].ColumnStyle = ColumnStyle.AutoWidth;
			t.AnchorPattern = AnchorPattern.TopInBottomOut;
			t.EnabledHandles = AdjustmentHandles.Move;
			diagram.Nodes.Add(t);
		}

		private void _btnNewIface_Click(object sender, EventArgs e)
		{
			TableNode t = new TableNode(diagram);
			t.Bounds = new RectangleF(10, 10, 30, diagram.TableCaptionHeight + 2);
			t.Style = new TableNodeStyle();
			t.Style.FontStyle = FontStyle.Italic;
			t.Caption = "Interface";
			t.Columns[0].ColumnStyle = ColumnStyle.AutoWidth;
			t.AnchorPattern = AnchorPattern.TopInBottomOut;
			t.Brush = new SolidBrush(Color.FromArgb(150, Color.LightSteelBlue));
			t.EnabledHandles = AdjustmentHandles.Move;
			diagram.Nodes.Add(t);
		}

		private void _btnNewEnum_Click(object sender, EventArgs e)
		{
			TableNode t = new TableNode(diagram);
			t.Bounds = new RectangleF(10, 10, 30, diagram.TableCaptionHeight + 2);
			t.Style = new TableNodeStyle();
			t.Style.FontStyle = FontStyle.Bold;
			t.Caption = "Enum";
			t.Columns[0].ColumnStyle = ColumnStyle.AutoWidth;
            t.Brush = new SolidBrush(Color.FromArgb(150, Color.FromArgb(206, 0, 0)));
			t.EnabledHandles = AdjustmentHandles.Move;
			diagram.Nodes.Add(t);
		}

		private void _btnNotation_Click(object sender, EventArgs e)
		{
			ShapeNode b = new ShapeNode(diagram);
			b.Bounds = new RectangleF(10, 10, 25, 15);

			b.Brush = new LinearGradientBrush(
                Color.FromArgb(100, Color.White), Color.FromArgb(192, 192, 192), 30);
			b.Text = "<note>";
			b.Pen = new Pen(Color.Black, 0);
			b.Pen.CompoundArray = new float[] { 0, 0.1f, 0.9f, 1 };

			b.TextFormat.Alignment = StringAlignment.Near;
			b.TextFormat.LineAlignment = StringAlignment.Near;

			diagram.Nodes.Add(b);
		}

		private void _menuUp_Click(object sender, EventArgs e)
		{
			if (_r > 0 && _r < _obj.RowCount && _obj.RowCount > 1)
			{
				string text = _obj[0, _r - 1].Text;
				_obj[0, _r - 1].Text = _obj[0, _r].Text;
				_obj[0, _r].Text = text;
			}
		}

		private void _menuDown_Click(object sender, EventArgs e)
		{
			if (_r < _obj.RowCount - 1)
			{
				string text = _obj[0, _r + 1].Text;
				_obj[0, _r + 1].Text = _obj[0, _r].Text;
				_obj[0, _r].Text = text;
			}
		}

		private void _menuInsert_Click(object sender, EventArgs e)
		{
			_obj.InsertRow(_r);
			_obj[0, _r].Text = "<item>";

			RectangleF rc = _obj.Bounds;
			rc.Height += _obj.RowHeight;
			_obj.Bounds = rc;
		}

		private void _menuDelete_Click(object sender, EventArgs e)
		{
			if (_r >= _obj.RowCount)
				return;

			_obj.DeleteRow(_r);

			RectangleF rc = _obj.Bounds;
			rc.Height -= _obj.RowHeight;
			_obj.Bounds = rc;
		}

		private void _btnDelete_Click(object sender, EventArgs e)
		{
			while (diagram.Selection.Items.Count > 0)
				diagram.Items.Remove(diagram.Selection.Items[0]);
		}

		private void _btnDerive_Click(object sender, EventArgs e)
		{
			diagram.LinkHeadShape = ArrowHeads.RevTriangle;
			diagram.LinkShape = LinkShape.Cascading;
			diagram.LinkSegments = 3;
			diagram.LinkCascadeOrientation = Orientation.Vertical;
		}

		private void _btnCompose_Click(object sender, EventArgs e)
		{
			diagram.LinkHeadShape = ArrowHeads.Rhombus;
			diagram.LinkShape = LinkShape.Cascading;
			diagram.LinkSegments = 3;
			diagram.LinkCascadeOrientation = Orientation.Horizontal;
		}

		private void _btnLinkNotation_Click(object sender, EventArgs e)
		{
			diagram.LinkHeadShape = ArrowHeads.None;
			diagram.LinkShape = LinkShape.Polyline;
			diagram.LinkSegments = 1;
		}

		private void DiagramForm_Load(object sender, EventArgs e)
		{
			diagram.DefaultShape = Shapes.Rectangle;
		}

		private void diagram_LinkCreated(object sender, LinkEventArgs e)
		{
			ShapeNode b = null;
			DiagramNode other = null;
			if (e.Link.Origin is ShapeNode)
			{
				b = (ShapeNode)e.Link.Origin;
				other = e.Link.Destination;
			}
			if (e.Link.Destination is ShapeNode)
			{
				b = (ShapeNode)e.Link.Destination;
				other = e.Link.Destination;
			}

			if (b != null)
				b.AttachTo(other, AttachToNode.TopLeft);
		}

		private void diagram_LinkDeleted(object sender, LinkEventArgs e)
		{
			ShapeNode b = null;
			DiagramNode other = null;
			if (e.Link.Origin is ShapeNode)
			{
				b = (ShapeNode)e.Link.Origin;
				other = e.Link.Destination;
			}
			if (e.Link.Destination is ShapeNode)
			{
				b = (ShapeNode)e.Link.Destination;
				other = e.Link.Destination;
			}

			if (b != null)
			{
				if (other.SubordinateGroup != null)
					other.SubordinateGroup.Detach(b);
			}
		}

		private void diagram_NodeClicked(object sender, NodeEventArgs e)
		{
			TableNode table = e.Node as TableNode;
			if (e.MouseButton == MouseButton.Right && table != null)
			{
				_obj = table;
				int row = 0, col = 0;
				table.CellFromPoint(e.MousePosition, ref row, ref col);
				_r = row;
				if (_r < 0)
					_r = _obj.RowCount;

				_cmenu.Show(diagramView, diagramView.DocToClient(e.MousePosition));
			}
		}


		public Diagram Diagram
		{
			get { return _diagram; }
			set
			{
				_diagram = value;
				if (_diagram.Data != "")
					diagram.LoadFromString(_diagram.Data);

				string s = diagramView.ZoomFactor.ToString();
				for (int i = 0; i < _cbZoom.Items.Count; i++)
					if ((string)_cbZoom.Items[i] == s)
					{
						_cbZoom.SelectedIndex = i;
						break;
					}
			}
		}


		private Diagram _diagram;
		private TableNode _obj;
		private int _r;
	}
}